@@ -14,7 +14,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# $Id: dio.tcl 1388916 2012-09-22 22:51:20Z mxmanghi $
+# $Id: dio.tcl 1444856 2013-02-11 16:39:17Z mxmanghi $
 
 catch {package require Tclx}
 package require Itcl
@@ -448,11 +448,11 @@ proc handle {interface args} {
 	$res destroy
 
 	if {$numrows} {
-            return [eval $this update $arrayName $args]
+            $this update $arrayName {*}$args
 	} else {
-            return [eval $this insert $myTable $arrayName] 
+            $this insert $myTable $arrayName
 	}
-
+        return 1
     }
 
     #
