@@ -14,7 +14,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# $Id: dio_Mysql.tcl 1372342 2012-08-13 09:35:09Z mxmanghi $
+# $Id: dio_Mysql.tcl 1523244 2013-09-14 14:05:00Z mxmanghi $
 
 package provide dio_Mysql 0.2
 
@@ -72,9 +72,12 @@ namespace eval DIO {
 #	    select is a 6 characters word, so let's see if the query is a select
 #
 	    set q [::string trim $req]
-	    set q [::string tolower $q]
-	    set q [::string range $q 0 5]
-	    if {[::string match select $q]} { set cmd mysqlsel }
+
+#	    set q [::string tolower $q]
+#	    set q [::string range $q 0 5]
+#	    if {[::string match select $q]} { set cmd mysqlsel }
+
+	    if {[regexp -nocase {^\(*\s*select\s+} $q]} { set cmd mysqlsel }
 
 	    set errorinfo ""
 	    if {[catch {$cmd $conn $req} error]} {
