@@ -14,7 +14,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# $Id: dio_Postgresql.tcl 963411 2010-07-12 18:12:14Z mxmanghi $
+# $Id: dio_Postgresql.tcl 1373643 2012-08-15 21:45:39Z mxmanghi $
 
 package provide dio_Postgresql 0.1
 
@@ -102,6 +102,65 @@ namespace eval DIO {
 	    return $conn
 	}
 
+
+	method makeDBFieldValue {table_name field_name val {convert_to {}}} {
+            if {[info exists specialFields(${table_name}@${field_name})]} {
+                switch $specialFields(${table_name}@${field_name}) {
+                    DATE {
+                            set secs [clock scan $val]
+                            set my_val [clock format $secs -format {%Y-%m-%d}]
+                            return "'$my_val'"
+                    }
+                    DATETIME {
+                            set secs [clock scan $val]
+                            set my_val [clock format $secs -format {%Y-%m-%d %T}]
+                            return "'$my_val'"
+                    }
+                    NOW {
+                        switch $convert_to {
+
+                            # we try to be coherent with the original purpose of this method whose
+                            # goal is to provide to the programmer a uniform way to handle timestamps. 
+                            # E.g.: Package session expects this case to return a timestamp in seconds
+                            # so that differences with timestamps returned by [clock seconds]
+                            # can be done and session expirations are computed consistently.
+                            # (Bug #53703)
+
+                            SECS {
+                                if {[::string compare $val "now"] == 0} {
+#					set	secs    [clock seconds]
+#					set	my_val  [clock format $secs -format {%Y%m%d%H%M%S}]
+#					return	$my_val
+                                    return [clock seconds]
+                                } else {
+                                    return  "extract(epoch from $field_name)"
+                                }
+                            }
+                            default {
+                                if {[::string compare $val, "now"] == 0} {
+                                    set secs [clock seconds]
+                                } else {
+                                    set secs [clock scan $val]
+                                }
+
+                                # this is kind of going back and forth from the same 
+                                # format,
+
+                                return "'[clock format $secs -format {%Y-%m-%d %T}]'"
+                            }
+                        }
+                    }
+                    default {
+                        # no special code for that type!!
+                        return "'[quote $val]'"
+                    }
+                }
+            } else {
+                    return "'[quote $val]'"
+            }
+	}
+
+
 	## If they change DBs, we need to close the connection and re-open it.
 	public variable db "" {
 	    if {[info exists conn]} {
