@@ -14,7 +14,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-# $Id: form.tcl 1370585 2012-08-07 23:14:36Z mxmanghi $
+# $Id: form.tcl 1518826 2013-08-29 21:20:38Z mxmanghi $
 #
 
 package require Itcl
@@ -35,7 +35,7 @@ package provide form 1.0
         # set the form method to be a post and the action to be
         # a refetching of the current page
         set arguments(method) post
-        set arguments(action) [env DOCUMENT_URI]
+        set arguments(action) [::rivet::env DOCUMENT_URI]
 
         # use $this for the type for form-global stuff like form arguments
         import_data form $this arguments $args
