@@ -14,12 +14,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# $Id: tclrivet.tcl 1300262 2012-03-13 18:05:13Z mxmanghi $
+# $Id: tclrivet.tcl 1518826 2013-08-29 21:20:38Z mxmanghi $
 
 package provide tclrivet 0.1
 
-load [file join [file dirname [info script]] .. .. \
-	  librivetparser[info sharedlibextension]]
+#load [file join [file dirname [info script]] .. .. \
+#	  librivetparser[info sharedlibextension]]
 
 proc include { filename } {
     set fl [ open $filename ]
