@@ -12,7 +12,7 @@
 ##              list attribute-value pairs to be given
 ##              to the <table> element tag
 ##
-## $Id: parray_table.tcl 1231782 2012-01-15 22:40:35Z mxmanghi $
+## $Id: parray_table.tcl 1523244 2013-09-14 14:05:00Z mxmanghi $
 ##
 ##
 
@@ -31,7 +31,8 @@ namespace eval ::rivet {
 	    puts "><thead><tr><th colspan=\"2\">$arrayName</th></tr></thead>"
         puts stdout "<tbody>"
         foreach name [lsort [array names array $pattern]] {
-            puts stdout [format "<tr><td>%s</td><td>%s</td></tr>" $name $array($name)]
+            puts stdout [format "<tr><td>%s</td><td>%s</td></tr>" [::rivet::escape_sgml_chars $name]    \
+                                                                  [::rivet::escape_sgml_chars $array($name)]]
         }
         puts stdout "</tbody></table>"
     }
