@@ -16,7 +16,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# $Id: runtests.tcl 1172805 2011-09-19 20:35:50Z mxmanghi $
+# $Id: runtests.tcl 1385194 2012-09-15 22:18:33Z mxmanghi $
 
 
 set auto_path [linsert $auto_path 0 [file join [file dirname [info script]] apachetest]]
@@ -29,13 +29,13 @@ proc runtests_usage {} {
 }
 
 proc get_httpd_version {httpd} {
-    catch {exec $httpd -v} raw_string
-    set version  [lindex [regexp -inline {([0-9]{1,}\.[0-9]{1,}\.[0-9]{1,})} $raw_string]  1]
-    if [string match "1.3.*" $version] {
-        return 1
-    } else {
-        return 2
-    }
+	catch {exec $httpd -v} raw_string
+	set version  [lindex [regexp -inline {([0-9]{1,}\.[0-9]{1,}\.[0-9]{1,})} $raw_string]  1]
+	if [string match "1.3.*" $version] {
+		return 1
+	} else {
+		return 2
+	}
 }
 
 if { [llength $argv] < 1 } {
@@ -50,9 +50,9 @@ package require apachetest
 
 if { [encoding system] eq "utf-8" } {
     puts stderr {
-	System encoding is utf-8 - this is known to cause problems
-	with the test environment!  Continuing with tests in 5 seconds
-	using the iso8859-1 encoding.
+        System encoding is utf-8 - this is known to cause problems
+        with the test environment!  Continuing with tests in 5 seconds
+        using the iso8859-1 encoding.
     }
     after 5000
 }
@@ -66,50 +66,50 @@ if { [catch {
 
 
 if {$httpd_version == 1} {
-    apachetest::need_modules {
-            {mod_log_config	config_log_module}
-            {mod_mime		mime_module}
-            {mod_negotiation	negotiation_module}
-            {mod_dir		dir_module}
-            {mod_auth		auth_module}
-            {mod_access		access_module}
-    }
+	apachetest::need_modules {
+		{mod_log_config		config_log_module}
+		{mod_mime		    mime_module}
+		{mod_negotiation	negotiation_module}
+		{mod_dir		    dir_module}
+		{mod_auth		    auth_module}
+		{mod_access		    access_module}
+	}
 } else {
-    apachetest::need_modules {
-            {mod_mime           mime_module}
-            {mod_negotiation    negotiation_module}
-            {mod_dir            dir_module}
-            {mod_log_config     log_config_module}
-            {mod_authz_host     authz_host_module}
-    }
+	apachetest::need_modules {
+		{mod_mime           mime_module}
+		{mod_negotiation    negotiation_module}
+		{mod_dir            dir_module}
+		{mod_log_config     log_config_module}
+		{mod_authz_host     authz_host_module}
+	}
 }
 
 apachetest::makeconf server.conf {
     LoadModule rivet_module [file join $CWD .. src/apache-$apachetest::httpd_version .libs mod_rivet[info sharedlibextension]]
 
-    User $username
-    Group $group
+# User and Group directives removed to ease dependency of test suite from the output of command 'id' (from which
+# the values for these directives were inferred (Bug #53396)
 
     <IfModule mod_mime.c>
-        TypesConfig $CWD/mime.types
-        AddLanguage en .en
-        AddLanguage it .it
-        AddLanguage es .es
-        AddType application/x-httpd-rivet .rvt
-        AddType application/x-rivet-tcl .tcl
+    TypesConfig $CWD/mime.types
+    AddLanguage en .en
+    AddLanguage it .it
+    AddLanguage es .es
+    AddType application/x-httpd-rivet .rvt
+    AddType application/x-rivet-tcl .tcl
     </IfModule>
 
 
     <IfDefine SERVERCONFTEST>
-        RivetServerConf BeforeScript 'puts "Page Header"'
-        RivetServerConf AfterScript 'puts "Page Footer"'
+    RivetServerConf BeforeScript 'puts "Page Header"'
+    RivetServerConf AfterScript 'puts "Page Footer"'
     </IfDefine>
 
     <IfDefine DIRTEST>
-        <Directory />
-            RivetDirConf BeforeScript 'puts "Page Header"'
-            RivetDirConf AfterScript 'puts "Page Footer"'
-        </Directory>
+    <Directory />
+    RivetDirConf BeforeScript 'puts "Page Header"'
+    RivetDirConf AfterScript 'puts "Page Footer"'
+    </Directory>
     </IfDefine>
 
     # We can use this to include our own stuff for each test.
@@ -129,10 +129,10 @@ set env(TCLLIBPATH) [file normalize [file join [file dirname [info script]] rive
 
 switch -exact -- [lindex $argv 1] {
     startserver {
-	apachetest::startserver
+	    apachetest::startserver
     }
     default {
-	set argv [lrange $argv 1 end]
-	source [file join . rivet.test]
+        set argv [lrange $argv 1 end]
+        source [file join . rivet.test]
     }
 }
